/************************************************************************
 *                                                                      *
 *  Program package 'lvq_pak':                                          *
 *                                                                      *
 *  knntest.c                                                           *
 *  -displays the recognition accuracy by knn test                      *
 *                                                                      *
 *  Version 2.1                                                         *
 *  Date: 9 Oct 1992                                                    *
 *                                                                      *
 *  NOTE: This program package is copyrighted in the sense that it      *
 *  may be used for scientific purposes. The package as a whole, or     *
 *  parts thereof, cannot be included or used in any commercial         *
 *  application without written permission granted by its producents.   *
 *  No programs contained in this package may be copied for commercial  *
 *  distribution.                                                       *
 *                                                                      *
 *  All comments  concerning this program package may be sent to the    *
 *  e-mail address 'lvq@cochlea.hut.fi'.                                *
 *                                                                      *
 ************************************************************************/

#include <stdio.h>
#include <float.h>
#include "lvq_pak.h"

void compute_knnaccuracy(struct entries *data, struct entries *codes, int knn)
{
  int *res;
  int *tot;
  int nol, noc, i, j;
  int *index;
  int *hits;
  int total, stotal;
  float diff, difference, *diffsf;
  struct entries *codetmp;

  /* Number of different labels in usage */
  nol = number_of_labels();

  /* Number of codebook vectors */
  noc = 0;
  codetmp = data;
  while (codetmp != NULL) {
    noc++;
    codetmp = codetmp->next;
  }

  hits = (int *) oalloc(sizeof(int)*nol);
  res = (int *) oalloc(sizeof(int)*nol);
  tot = (int *) oalloc(sizeof(int)*nol);
  if (knn < 1) knn = 1;
  index = (int *) oalloc(sizeof(int)*knn);
  diffsf = (float *) oalloc(sizeof(float)*knn);

  /* Initialize all tables */
  for (i = 0; i < nol; i++) {
    res[i] = 0;
    tot[i] = 0;
  }
  stotal = 0;
  total = 0;

  /* Scan all input entries */
  while (data != NULL) {

    codetmp = codes;
    for (i = 0; i < knn; i++)
      diffsf[i] = FLT_MAX;

    /* Compare all code entries against the input entry */
    while (codetmp != NULL) {

      difference = 0.0;

      /* Compute the distance between the codebook and input entries */
      for (i = 0; i < data->dimension; i++) {
	diff = data->points[i] - codetmp->points[i];
	difference += diff * diff;
        if (difference > diffsf[knn-1]) break;
      }

      /* If distance is smaller than previous distances */
      i = 0;
      while ((i < knn) && (difference > diffsf[i]))
	i++;
      if (i < knn) {
	j = knn-1;
	while (j > i) {
	  diffsf[j] = diffsf[j-1];
	  index[j] = index[j-1];
	  j--;
	}
	diffsf[i] = difference;
	index[i] = codetmp->index;
      }

      /* Take the next codebook entry */
      codetmp = codetmp->next;
    }

    /* If classification was correct */
    for (i = 0; i < nol; i++) {
      hits[i] = 0;
    }
    for (i = 0; i < knn; i++) {
      hits[index[i]]++;
    }
    j = 0;
    for (i = 0; i < nol; i++) {
      if (i != data->index)
        if (j < hits[i])
          j = hits[i];
    }
    if (hits[data->index] > j) {
      /* Number of correct classifications */
      stotal++;

      /* Number of correct classifications in that class */
      res[data->index]++;
    }
     
    /* Total number of entries in that class */
    tot[data->index]++;

    /* Total number of entries */
    total++;

    /* Take the next input entry */
    data = data->next;

    if (verbose(-1) > 0)
      mprint((long) noc--);
  }
  if (verbose(-1) > 0)
    mprint((long) 0);
  if (verbose(-1) > 0)
    fprintf(stdout, "\n");

  fprintf(stdout, "\nRecognition accuracy:\n\n");
  for (i = 0; i < nol; i++) {
    fprintf(stdout, "%14s: ", find_conv_to_lab(i));
    fprintf(stdout, "%6.2f %%\n", 100.0 * (float) res[i] / tot[i]);
  }
  fprintf(stdout, "\nTotal accuracy: %6.2f %%\n\n",
	  100.0 * (float) stotal / total);
}

main(int argc, char **argv)
{
  int knn;
  char *in_data_file;
  char *in_code_file;
  struct entries *data, *codes;

  in_data_file = extract_parameter(argc, argv, IN_DATA_FILE, ALWAYS);
  in_code_file = extract_parameter(argc, argv, IN_CODE_FILE, ALWAYS);
  verbose((int) oatoi(extract_parameter(argc, argv, VERBOSE, OPTION), 1));
  knn = (int) oatoi(extract_parameter(argc, argv, KNN_NEIGHBORS, OPTION), 5);

  if (verbose(-1) > 1)
    fprintf(stdout, "Input entries are read from file %s\n", in_data_file);
  data = read_entries(in_data_file);
  if (verbose(-1) > 1)
    fprintf(stdout, "Codebook entries are read from file %s\n", in_code_file);
  codes = read_entries(in_code_file);

  if (data->dimension != codes->dimension) {
    errormsg("Data and codebook vectors have different dimensions");
    exit(0);
  }

  compute_knnaccuracy(data, codes, knn);

  return(0);
}
